@@ -19,7 +19,7 @@ use holochain_state::prelude::*;
 use holochain_types::test_utils::fake_cell_id;
 use holochain_wasm_test_utils::TestWasm;
 use holochain_websocket::WebsocketSender;
-use kitsune_p2p_types::dependencies::lair_keystore_api::LairError;
+use kitsune_p2p_types::dependencies::legacy_lair_api::LairError;
 use maplit::hashset;
 use matches::assert_matches;
 
@@ -217,8 +217,6 @@ async fn can_set_fake_state() {
 
 #[tokio::test(flavor = "multi_thread")]
 async fn proxy_tls_with_test_keystore() {
-    use ghost_actor::GhostControlSender;
-
     observability::test_run().ok();
 
     let keystore1 = spawn_test_keystore().await.unwrap();
@@ -228,13 +226,13 @@ async fn proxy_tls_with_test_keystore() {
         panic!("{:#?}", e);
     }
 
-    let _ = keystore1.ghost_actor_shutdown_immediate().await;
-    let _ = keystore2.ghost_actor_shutdown_immediate().await;
+    let _ = keystore1.shutdown().await;
+    let _ = keystore2.shutdown().await;
 }
 
 async fn proxy_tls_inner(
-    keystore1: KeystoreSender,
-    keystore2: KeystoreSender,
+    keystore1: MetaLairClient,
+    keystore2: MetaLairClient,
 ) -> anyhow::Result<()> {
     use ghost_actor::GhostControlSender;
     use kitsune_p2p::dependencies::*;
@@ -509,7 +507,7 @@ async fn make_signing_call(client: &mut WebsocketSender, cell: &SweetCell) -> Ap
 /// to fail.
 ///
 /// This test was written making the assumption that we could swap out the
-/// KeystoreSender for each Cell at runtime, but given our current concurrency
+/// MetaLairClient for each Cell at runtime, but given our current concurrency
 /// model which puts each Cell in an Arc, this is not possible.
 /// In order to implement this test, we should probably have the "crude mock
 /// keystore" listen on a channel which toggles its behavior from always-correct
@@ -715,33 +713,6 @@ async fn test_cells_disable_on_validation_panic() {
     );
 }
 
-#[tokio::test(flavor = "multi_thread")]
-async fn test_cells_disable_on_validation_error() {
-    observability::test_run().ok();
-    let bad_zome =
-        InlineZome::new_unique(Vec::new()).callback("validate", |_api, _data: ValidateData| {
-            InlineZomeResult::<ValidateResult>::Err(InlineZomeError::TestError(
-                "intentional error".into(),
-            ))
-        });
-
-    let mut conductor = SweetConductor::from_standard_config().await;
-
-    // This may be an error, depending on if validation runs before or after
-    // the app is enabled. Proceed in either case.
-    let _ = common_genesis_test_app(&mut conductor, bad_zome).await;
-
-    // - Ensure that the app was disabled because one Cell had a validation error
-    //   (while publishing genesis elements)
-    assert_eq_retry_10s!(
-        {
-            let state = conductor.get_state_from_handle().await.unwrap();
-            (state.enabled_apps().count(), state.stopped_apps().count())
-        },
-        (0, 1)
-    );
-}
-
 #[tokio::test(flavor = "multi_thread")]
 async fn test_installation_fails_if_genesis_self_check_is_invalid() {
     observability::test_run().ok();
