@@ -56,6 +56,7 @@ use super::CellError;
 use super::Conductor;
 use crate::conductor::p2p_agent_store::get_single_agent_info;
 use crate::conductor::p2p_agent_store::query_peer_density;
+use crate::conductor::p2p_agent_store::P2pBatch;
 use crate::conductor::p2p_metrics::put_metric_datum;
 use crate::conductor::p2p_metrics::query_metrics;
 use crate::core::ribosome::real_ribosome::RealRibosome;
@@ -67,6 +68,7 @@ use holochain_conductor_api::conductor::EnvironmentRootPath;
 use holochain_conductor_api::AppStatusFilter;
 use holochain_conductor_api::InstalledAppInfo;
 use holochain_conductor_api::JsonDump;
+use holochain_keystore::MetaLairClient;
 use holochain_p2p::event::HolochainP2pEvent;
 use holochain_p2p::event::HolochainP2pEvent::*;
 use holochain_p2p::DnaHashExt;
@@ -126,7 +128,7 @@ pub trait ConductorHandleT: Send + Sync {
     /// List the app interfaces currently install.
     async fn list_app_interfaces(&self) -> ConductorResult<Vec<u16>>;
 
-    /// Install a [Dna] in this Conductor
+    /// Install a [DnaFile] in this Conductor
     async fn register_dna(&self, dna: DnaFile) -> ConductorResult<()>;
 
     /// Get the list of hashes of installed Dnas in this Conductor
@@ -175,7 +177,7 @@ pub trait ConductorHandleT: Send + Sync {
     fn shutdown(&self);
 
     /// Request access to this conductor's keystore
-    fn keystore(&self) -> &KeystoreSender;
+    fn keystore(&self) -> &MetaLairClient;
 
     /// Request access to this conductor's networking handle
     fn holochain_p2p(&self) -> &holochain_p2p::HolochainP2pRef;
@@ -382,7 +384,7 @@ pub trait ConductorHandleT: Send + Sync {
 #[derive(From)]
 pub struct ConductorHandleImpl<DS: DnaStore + 'static> {
     pub(super) conductor: Conductor<DS>,
-    pub(super) keystore: KeystoreSender,
+    pub(super) keystore: MetaLairClient,
     pub(super) holochain_p2p: holochain_p2p::HolochainP2pRef,
 
     /// The root environment directory where all environments are created
@@ -397,6 +399,10 @@ pub struct ConductorHandleImpl<DS: DnaStore + 'static> {
     /// Database sync level
     pub(super) db_sync_level: DbSyncLevel,
 
+    /// The batch sender for writes to the p2p database.
+    pub(super) p2p_batch_senders:
+        Arc<parking_lot::Mutex<HashMap<Arc<KitsuneSpace>, tokio::sync::mpsc::Sender<P2pBatch>>>>,
+
     // Testing:
     #[cfg(any(test, feature = "test_utils"))]
     /// All conductors should skip publishing.
@@ -509,10 +515,18 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
             PutAgentInfoSigned {
                 peer_data, respond, ..
             } => {
-                let env = { self.p2p_env(space) };
-                let res = inject_agent_infos(env, peer_data.iter())
-                    .await
-                    .map_err(holochain_p2p::HolochainP2pError::other);
+                let sender = self.p2p_batch_sender(space);
+                let (result_sender, response) = tokio::sync::oneshot::channel();
+                let _ = sender
+                    .send(P2pBatch {
+                        peer_data,
+                        result_sender,
+                    })
+                    .await;
+                let res = match response.await {
+                    Ok(r) => r.map_err(holochain_p2p::HolochainP2pError::other),
+                    Err(e) => Err(holochain_p2p::HolochainP2pError::other(e)),
+                };
                 respond.respond(Ok(async move { res }.boxed().into()));
             }
             GetAgentInfoSigned {
@@ -609,7 +623,7 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
                 data,
                 ..
             } => {
-                let signature = to_agent.sign_raw(self.keystore(), &data).await?;
+                let signature = to_agent.sign_raw(self.keystore(), data.into()).await?;
                 respond.respond(Ok(async move { Ok(signature) }.boxed().into()));
             }
             HolochainP2pEvent::CallRemote { .. }
@@ -730,7 +744,7 @@ impl<DS: DnaStore + 'static> ConductorHandleT for ConductorHandleImpl<DS> {
         self.conductor.shutdown()
     }
 
-    fn keystore(&self) -> &KeystoreSender {
+    fn keystore(&self) -> &MetaLairClient {
         &self.keystore
     }
 
@@ -1297,7 +1311,7 @@ impl<DS: DnaStore + 'static> ConductorHandleImpl<DS> {
 
         let pending_cells: Vec<(CellId, HolochainP2pCell)> = self
             .conductor
-            .pending_cells()
+            .mark_pending_cells_as_joining()
             .into_iter()
             .map(|(id, cell)| (id, cell.holochain_p2p_cell().clone()))
             .collect();
@@ -1307,13 +1321,13 @@ impl<DS: DnaStore + 'static> ConductorHandleImpl<DS> {
                 match tokio::time::timeout(JOIN_NETWORK_TIMEOUT, network.join()).await {
                     Ok(Err(e)) => {
                         tracing::info!(error = ?e, cell_id = ?cell_id, "Error while trying to join the network");
-                        None
+                        Err(cell_id)
                     }
                     Err(_) => {
                         tracing::info!(cell_id = ?cell_id, "Timed out trying to join the network");
-                        None
+                        Err(cell_id)
                     }
-                    Ok(Ok(_)) => Some(cell_id),
+                    Ok(Ok(_)) => Ok(cell_id),
                 }
             });
 
@@ -1322,13 +1336,21 @@ impl<DS: DnaStore + 'static> ConductorHandleImpl<DS> {
             .collect()
             .await;
 
-        let cell_ids: Vec<_> = maybes.into_iter().flatten().collect();
+        let (cell_ids, failed_joins): (Vec<_>, Vec<_>) =
+            maybes.into_iter().partition(Result::is_ok);
+
+        // These unwraps are both safe because of the partition.
+        let cell_ids: Vec<_> = cell_ids.into_iter().map(Result::unwrap).collect();
+        let failed_joins: Vec<_> = failed_joins.into_iter().map(Result::unwrap_err).collect();
 
         // Update the status of the cells which were able to join the network
         // (may or may not be all cells which were added)
         self.conductor
             .update_cell_status(cell_ids.as_slice(), CellStatus::Joined);
 
+        self.conductor
+            .update_cell_status(failed_joins.as_slice(), CellStatus::PendingJoin);
+
         cell_ids
     }
 
@@ -1351,6 +1373,22 @@ impl<DS: DnaStore + 'static> ConductorHandleImpl<DS> {
             .clone()
     }
 
+    pub(super) fn p2p_batch_sender(
+        &self,
+        space: Arc<KitsuneSpace>,
+    ) -> tokio::sync::mpsc::Sender<P2pBatch> {
+        let mut p2p_env = self.p2p_batch_senders.lock();
+        p2p_env
+            .entry(space.clone())
+            .or_insert_with(|| {
+                let (tx, rx) = tokio::sync::mpsc::channel(100);
+                let env = { self.p2p_env(space) };
+                tokio::spawn(p2p_agent_store::p2p_put_all_batch(env, rx));
+                tx
+            })
+            .clone()
+    }
+
     pub(super) fn p2p_metrics_env(&self, space: Arc<KitsuneSpace>) -> EnvWrite {
         let mut p2p_metrics_env = self.p2p_metrics_env.lock();
         let db_sync_level = self.db_sync_level;
