@@ -67,7 +67,7 @@ impl Invocation for ValidateInvocation {
         }
         match self.element.entry().as_option() {
             Some(Entry::Agent(_)) => fns.push("agent".into()),
-            Some(Entry::App(_)) => {
+            Some(Entry::App(_)) | Some(Entry::CounterSign(_, _)) => {
                 fns.push("entry".into());
                 if let Some(EntryDefId::App(entry_def_id)) = self.entry_def_id.clone() {
                     fns.push(entry_def_id);
@@ -290,7 +290,6 @@ mod slow_tests {
     use crate::fixt::*;
     use ::fixt::prelude::*;
     use holo_hash::fixt::AgentPubKeyFixturator;
-    use holochain_state::host_fn_workspace::HostFnWorkspace;
     use holochain_state::source_chain::SourceChainResult;
     use holochain_types::prelude::*;
     use holochain_wasm_test_utils::TestWasm;
@@ -372,17 +371,7 @@ mod slow_tests {
 
     #[tokio::test(flavor = "multi_thread")]
     async fn pass_validate_test<'a>() {
-        let test_env = holochain_state::test_utils::test_cell_env();
-        let test_cache = holochain_state::test_utils::test_cache_env();
-        let env = test_env.env();
-        let author = fake_agent_pubkey_1();
-        crate::test_utils::fake_genesis(env.clone()).await.unwrap();
-        let workspace = HostFnWorkspace::new(env.clone(), test_cache.env(), author)
-            .await
-            .unwrap();
-        // test workspace boilerplate
-        let mut host_access = fixt!(ZomeCallHostAccess);
-        host_access.workspace = workspace.clone();
+        let host_access = fixt!(ZomeCallHostAccess, Predictable);
 
         let output: HeaderHash =
             crate::call_test_ribosome!(host_access, TestWasm::Validate, "always_validates", ())
@@ -390,7 +379,7 @@ mod slow_tests {
 
         // the chain head should be the committed entry header
         let chain_head = tokio_helper::block_forever_on(async move {
-            SourceChainResult::Ok(workspace.source_chain().chain_head()?.0)
+            SourceChainResult::Ok(host_access.workspace.source_chain().chain_head()?.0)
         })
         .unwrap();
 
@@ -399,18 +388,7 @@ mod slow_tests {
 
     #[tokio::test(flavor = "multi_thread")]
     async fn fail_validate_test<'a>() {
-        let test_env = holochain_state::test_utils::test_cell_env();
-        let test_cache = holochain_state::test_utils::test_cache_env();
-        let env = test_env.env();
-        let author = fake_agent_pubkey_1();
-        crate::test_utils::fake_genesis(env.clone()).await.unwrap();
-        let workspace = HostFnWorkspace::new(env.clone(), test_cache.env(), author)
-            .await
-            .unwrap();
-        // test workspace boilerplate
-
-        let mut host_access = fixt!(ZomeCallHostAccess);
-        host_access.workspace = workspace.clone();
+        let host_access = fixt!(ZomeCallHostAccess, Predictable);
 
         let output: HeaderHash =
             crate::call_test_ribosome!(host_access, TestWasm::Validate, "never_validates", ())
@@ -418,7 +396,7 @@ mod slow_tests {
 
         // the chain head should be the committed entry header
         let chain_head = tokio_helper::block_forever_on(async move {
-            SourceChainResult::Ok(workspace.source_chain().chain_head()?.0)
+            SourceChainResult::Ok(host_access.workspace.source_chain().chain_head()?.0)
         })
         .unwrap();
 
