@@ -5,36 +5,35 @@ use holochain_types::prelude::*;
 use holochain_wasmer_host::prelude::WasmError;
 use std::sync::Arc;
 use crate::core::ribosome::HostFnAccess;
+use futures::future::join_all;
 
 #[allow(clippy::extra_unused_lifetimes)]
 pub fn get_details<'a>(
     _ribosome: Arc<impl RibosomeT>,
     call_context: Arc<CallContext>,
-    input: GetInput,
-) -> Result<Option<Details>, WasmError> {
+    inputs: Vec<GetInput>,
+) -> Result<Vec<Option<Details>>, WasmError> {
     match HostFnAccess::from(&call_context.host_context()) {
         HostFnAccess{ read_workspace: Permission::Allow, .. } => {
-            let GetInput {
-                any_dht_hash,
-                get_options,
-            } = input;
-
-    // Get the network from the context
-    let network = call_context.host_context.network().clone();
-
-    // timeouts must be handled by the network
-    tokio_helper::block_forever_on(async move {
-        let workspace = call_context.host_context.workspace();
-        let mut cascade = Cascade::from_workspace_network(workspace, network);
-        let maybe_details = cascade
-            .get_details(any_dht_hash, get_options)
-            .await
-            .map_err(|cascade_error| WasmError::Host(cascade_error.to_string()))?;
-        Ok(maybe_details)
-    })
-    },
-    _ => unreachable!(),
-}
+            let results: Vec<Result<Option<Details>, _>> = tokio_helper::block_forever_on(async move {
+                join_all(inputs.into_iter().map(|input| {
+                    async {
+                        let GetInput {
+                            any_dht_hash,
+                            get_options,
+                        } = input;
+                        Cascade::from_workspace_network(
+                            call_context.host_context.workspace(),
+                            call_context.host_context.network().to_owned(),
+                        ).get_details(any_dht_hash, get_options).await
+                    }
+                })).await
+            });
+            let results: Result<Vec<_>, _> = results.into_iter().map(|result| result.map_err(|cascade_error| WasmError::Host(cascade_error.to_string()))).collect();
+            Ok(results?)
+        },
+        _ => unreachable!(),
+    }
 }
 
 #[cfg(test)]
@@ -43,30 +42,18 @@ pub mod wasm_test {
     use crate::fixt::ZomeCallHostAccessFixturator;
     use ::fixt::prelude::*;
     use hdk::prelude::*;
-    use holochain_state::host_fn_workspace::HostFnWorkspace;
     use holochain_wasm_test_utils::TestWasm;
 
     #[tokio::test(flavor = "multi_thread")]
     async fn ribosome_get_details_test<'a>() {
         observability::test_run().ok();
-        let test_env = holochain_state::test_utils::test_cell_env();
-        let test_cache = holochain_state::test_utils::test_cache_env();
-        let env = test_env.env();
-        let author = fake_agent_pubkey_1();
-        crate::test_utils::fake_genesis(env.clone())
-            .await
-            .unwrap();
-        let workspace = HostFnWorkspace::new(env.clone(), test_cache.env(), author).await.unwrap();
-
-
-        let mut host_access = fixt!(ZomeCallHostAccess);
-        host_access.workspace = workspace.clone();
+        let host_access = fixt!(ZomeCallHostAccess, Predictable);
 
         // simple replica of the internal type for the TestWasm::Crud entry
         #[derive(Clone, Copy, Serialize, Deserialize, SerializedBytes, Debug, PartialEq)]
         struct CounTree(u32);
 
-        let check = |details: Option<Details>, count, delete| match details {
+        let check = |details: &Option<Details>, count, delete| match details {
             Some(Details::Element(ref element_details)) => {
                 match element_details.element.entry().to_app_option::<CounTree>() {
                     Ok(Some(CounTree(u))) => assert_eq!(u, count),
@@ -77,7 +64,7 @@ pub mod wasm_test {
             _ => panic!("no element"),
         };
 
-        let check_entry = |details: Option<Details>, count, update, delete, line| match details {
+        let check_entry = |details: &Option<Details>, count, update, delete, line| match details {
             Some(Details::Entry(ref entry_details)) => {
                 match entry_details.entry {
                     Entry::App(ref eb) => {
@@ -103,13 +90,15 @@ pub mod wasm_test {
             crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_hash", CounTree(2)).unwrap();
 
         let zero_a: HeaderHash = crate::call_test_ribosome!(host_access, TestWasm::Crud, "new", ()).unwrap();
+        let header_details_0: Vec<Option<Details>> = crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", vec![zero_a.clone()]).unwrap();
+        let entry_details_0: Vec<Option<Details>> = crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", vec![zero_hash.clone()]).unwrap();
         check(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", zero_a).unwrap(),
+            &header_details_0[0],
             0,
             0,
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", zero_hash).unwrap(),
+            &entry_details_0[0],
             0,
             0,
             0,
@@ -118,25 +107,27 @@ pub mod wasm_test {
 
         let one_a: HeaderHash =
             crate::call_test_ribosome!(host_access, TestWasm::Crud, "inc", zero_a).unwrap();
+        let header_details_1: Vec<Option<Details>> = crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", vec![zero_a.clone(), one_a.clone()]).unwrap();
+        let entry_details_1: Vec<Option<Details>> = crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", vec![zero_hash.clone(), one_hash.clone()]).unwrap();
         check(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", zero_a).unwrap(),
+            &header_details_1[0],
             0,
             0,
         );
         check(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", one_a).unwrap(),
+            &header_details_1[1],
             1,
             0,
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", zero_hash).unwrap(),
+            &entry_details_1[0],
             0,
             1,
             0,
             line!(),
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", one_hash).unwrap(),
+            &entry_details_1[1],
             1,
             0,
             0,
@@ -145,25 +136,27 @@ pub mod wasm_test {
 
         let one_b: HeaderHash =
             crate::call_test_ribosome!(host_access, TestWasm::Crud, "inc", zero_a).unwrap();
+        let header_details_2: Vec<Option<Details>> = crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", vec![zero_a, one_b.clone()]).unwrap();
+        let entry_details_2: Vec<Option<Details>> = crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", vec![zero_hash.clone(), one_hash.clone()]).unwrap();
         check(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", zero_a).unwrap(),
+            &header_details_2[0],
             0,
             0,
         );
         check(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", one_b).unwrap(),
+            &header_details_2[1],
             1,
             0,
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", zero_hash).unwrap(),
+            &entry_details_2[0],
             0,
             2,
             0,
             line!(),
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", one_hash).unwrap(),
+            &entry_details_2[1],
             1,
             0,
             0,
@@ -171,32 +164,34 @@ pub mod wasm_test {
         );
 
         let two: HeaderHash = crate::call_test_ribosome!(host_access, TestWasm::Crud, "inc", one_b).unwrap();
+        let header_details_3: Vec<Option<Details>> = crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", vec![one_b.clone(), two]).unwrap();
+        let entry_details_3: Vec<Option<Details>> = crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", vec![zero_hash.clone(), one_hash.clone(), two_hash.clone()]).unwrap();
         check(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", one_b).unwrap(),
+            &header_details_3[0],
             1,
             0,
         );
         check(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", two).unwrap(),
+            &header_details_3[1],
             2,
             0,
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", zero_hash).unwrap(),
+            &entry_details_3[0],
             0,
             2,
             0,
             line!(),
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", one_hash).unwrap(),
+            &entry_details_3[1],
             1,
             1,
             0,
             line!(),
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", two_hash).unwrap(),
+            &entry_details_3[2],
             2,
             0,
             0,
@@ -205,42 +200,42 @@ pub mod wasm_test {
 
         let zero_b: HeaderHash =
             crate::call_test_ribosome!(host_access, TestWasm::Crud, "dec", one_a).unwrap();
+        let header_details_4: Vec<Option<Details>> = crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", vec![one_a, one_b, zero_b]).unwrap();
+        let entry_details_4: Vec<Option<Details>> = crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", vec![zero_hash, one_hash, two_hash]).unwrap();
         check(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", one_a).unwrap(),
+            &header_details_4[0],
             1,
             1,
         );
         check(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", one_b).unwrap(),
+            &header_details_4[1],
             1,
             0,
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", zero_hash).unwrap(),
+            &entry_details_4[0],
             0,
             2,
             0,
             line!(),
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", one_hash).unwrap(),
+            &entry_details_4[1],
             1,
             1,
             1,
             line!(),
         );
         check_entry(
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "entry_details", two_hash).unwrap(),
+            &entry_details_4[2],
             2,
             0,
             0,
             line!(),
         );
 
-        let zero_b_details: Option<Details> =
-            crate::call_test_ribosome!(host_access, TestWasm::Crud, "header_details", zero_b).unwrap();
-        match zero_b_details {
-            Some(Details::Element(element_details)) => {
+        match header_details_4[2] {
+            Some(Details::Element(ref element_details)) => {
                 match element_details.element.entry().as_option() {
                     None => {
                         // this is the delete so it should be none
