@@ -6,46 +6,46 @@ use holochain_types::prelude::*;
 use holochain_wasmer_host::prelude::WasmError;
 use std::sync::Arc;
 use crate::core::ribosome::HostFnAccess;
+use futures::future::join_all;
 
 #[allow(clippy::extra_unused_lifetimes)]
 pub fn get_links<'a>(
     ribosome: Arc<impl RibosomeT>,
     call_context: Arc<CallContext>,
-    input: GetLinksInput,
-) -> Result<Links, WasmError> {
+    inputs: Vec<GetLinksInput>,
+) -> Result<Vec<Links>, WasmError> {
     match HostFnAccess::from(&call_context.host_context()) {
         HostFnAccess{ read_workspace: Permission::Allow, .. } => {
-            let GetLinksInput {
-                base_address,
-                tag_prefix,
-            } = input;
-
-            // Get zome id
-            let zome_id = ribosome
-                .zome_to_id(&call_context.zome)
-                .expect("Failed to get ID for current zome.");
-
-    // Get the network from the context
-    let network = call_context.host_context.network().clone();
-
-    tokio_helper::block_forever_on(async move {
-        // Create the key
-        let key = WireLinkKey {
-            base: base_address,
-            zome_id,
-            tag: tag_prefix,
-        };
-        let workspace = call_context.host_context.workspace();
-        let mut cascade = Cascade::from_workspace_network(workspace, network);
-
-                // Get the links from the dht
-                let links = cascade
-                    .dht_get_links(key, GetLinksOptions::default())
-                    .await
-                    .map_err(|cascade_error| WasmError::Host(cascade_error.to_string()))?;
-
-                Ok(links.into())
-            })
+            let results: Vec<Result<Vec<Link>, _>> = tokio_helper::block_forever_on(async move {
+                join_all(inputs.into_iter().map(|input| {
+                    async {
+                        let GetLinksInput {
+                            base_address,
+                            tag_prefix,
+                        } = input;
+                        let zome_id = ribosome
+                            .zome_to_id(&call_context.zome)
+                            .expect("Failed to get ID for current zome.");
+                        let key = WireLinkKey {
+                            base: base_address,
+                            zome_id,
+                            tag: tag_prefix,
+                        };
+                        Cascade::from_workspace_network(
+                            call_context.host_context.workspace(),
+                            call_context.host_context.network().to_owned(),
+                        ).dht_get_links(key, GetLinksOptions::default()).await
+                    }
+                }
+                )).await
+            });
+            let results: Result<Vec<_>, _> = results.into_iter().map(|result|
+                match result {
+                    Ok(links_vec) => Ok(links_vec.into()),
+                    Err(cascade_error) => Err(WasmError::Host(cascade_error.to_string())),
+                }
+            ).collect();
+            Ok(results?)
         },
         _ => unreachable!(),
     }
@@ -61,7 +61,6 @@ pub mod slow_tests {
     use crate::test_utils::WaitOps;
     use ::fixt::prelude::*;
     use hdk::prelude::*;
-    use holochain_state::host_fn_workspace::HostFnWorkspace;
     use holochain_test_wasm_common::*;
     use holochain_wasm_test_utils::TestWasm;
     use matches::assert_matches;
@@ -69,14 +68,7 @@ pub mod slow_tests {
     #[tokio::test(flavor = "multi_thread")]
     async fn ribosome_entry_hash_path_children() {
         observability::test_run().ok();
-        let test_env = holochain_state::test_utils::test_cell_env();
-        let test_cache = holochain_state::test_utils::test_cache_env();
-        let env = test_env.env();
-        let author = fake_agent_pubkey_1();
-        crate::test_utils::fake_genesis(env.clone()).await.unwrap();
-        let workspace = HostFnWorkspace::new(env.clone(), test_cache.env(), author).await.unwrap();
-        let mut host_access = fixt!(ZomeCallHostAccess);
-        host_access.workspace = workspace;
+        let host_access = fixt!(ZomeCallHostAccess, Predictable);
 
         // ensure foo.bar twice to ensure idempotency
         let _: () = crate::call_test_ribosome!(
@@ -138,14 +130,7 @@ pub mod slow_tests {
 
     #[tokio::test(flavor = "multi_thread")]
     async fn hash_path_anchor_get_anchor() {
-        let test_env = holochain_state::test_utils::test_cell_env();
-        let test_cache = holochain_state::test_utils::test_cache_env();
-        let env = test_env.env();
-        let author = fake_agent_pubkey_1();
-        crate::test_utils::fake_genesis(env.clone()).await.unwrap();
-        let workspace = HostFnWorkspace::new(env.clone(), test_cache.env(), author).await.unwrap();
-        let mut host_access = fixt!(ZomeCallHostAccess);
-        host_access.workspace = workspace;
+        let host_access = fixt!(ZomeCallHostAccess, Predictable);
 
         // anchor foo bar
         let anchor_address_one: EntryHash = crate::call_test_ribosome!(
@@ -246,6 +231,72 @@ pub mod slow_tests {
         );
     }
 
+    #[tokio::test(flavor = "multi_thread")]
+    async fn multi_get_links() {
+        observability::test_run().ok();
+        let host_access = fixt!(ZomeCallHostAccess, Predictable);
+        let _: HeaderHash = crate::call_test_ribosome!(
+            host_access,
+            TestWasm::Link,
+            "create_link",
+            ()
+        ).unwrap();
+        let _: HeaderHash = crate::call_test_ribosome!(
+            host_access,
+            TestWasm::Link,
+            "create_back_link",
+            ()
+        ).unwrap();
+        let forward_links: Links = crate::call_test_ribosome!(
+            host_access,
+            TestWasm::Link,
+            "get_links",
+            ()
+        ).unwrap();
+        let back_links: Links = crate::call_test_ribosome!(
+            host_access,
+            TestWasm::Link,
+            "get_back_links",
+            ()
+        ).unwrap();
+        let links_bidi: Vec<Links> = crate::call_test_ribosome!(
+            host_access,
+            TestWasm::Link,
+            "get_links_bidi",
+            ()
+        ).unwrap();
+
+        assert_eq!(
+            links_bidi,
+            vec![forward_links, back_links],
+        );
+
+        let forward_link_details: LinkDetails = crate::call_test_ribosome!(
+            host_access,
+            TestWasm::Link,
+            "get_link_details",
+            ()
+        ).unwrap();
+
+        let back_link_details: LinkDetails = crate::call_test_ribosome!(
+            host_access,
+            TestWasm::Link,
+            "get_back_link_details",
+            ()
+        ).unwrap();
+
+        let link_details_bidi: Vec<LinkDetails> = crate::call_test_ribosome!(
+            host_access,
+            TestWasm::Link,
+            "get_link_details_bidi",
+            ()
+        ).unwrap();
+        assert_eq!(
+            link_details_bidi,
+            vec![forward_link_details, back_link_details],
+        );
+    }
+
     #[tokio::test(flavor = "multi_thread")]
     async fn dup_path_test() {
         observability::test_run().ok();
