@@ -636,36 +636,77 @@ impl RibosomeT for RealRibosome {
 #[cfg(test)]
 #[cfg(feature = "slow_tests")]
 pub mod wasm_test {
-    use crate::fixt::ZomeCallHostAccessFixturator;
+    use crate::conductor::ConductorBuilder;
+    use crate::core::ribosome::MockDnaStore;
+    use crate::core::ribosome::ZomeCall;
+    use crate::sweettest::SweetConductor;
+    use crate::sweettest::SweetDnaFile;
     use ::fixt::prelude::*;
     use hdk::prelude::*;
-    use holochain_state::host_fn_workspace::HostFnWorkspace;
+    use holochain_types::prelude::AgentPubKeyFixturator;
     use holochain_wasm_test_utils::TestWasm;
 
     #[tokio::test(flavor = "multi_thread")]
     /// Basic checks that we can call externs internally and externally the way we want using the
     /// hdk macros rather than low level rust extern syntax.
     async fn ribosome_extern_test() {
-        let test_env = holochain_state::test_utils::test_cell_env();
-        let test_cache = holochain_state::test_utils::test_cache_env();
-        let env = test_env.env();
-        let author = fake_agent_pubkey_1();
-        crate::test_utils::fake_genesis(env.clone()).await.unwrap();
-        let workspace = HostFnWorkspace::new(env.clone(), test_cache.env(), author)
+        observability::test_run().ok();
+
+        let (dna_file, _) = SweetDnaFile::unique_from_test_wasms(vec![TestWasm::HdkExtern])
+            .await
+            .unwrap();
+        let alice_pubkey = fixt!(AgentPubKey, Predictable, 0);
+        let bob_pubkey = fixt!(AgentPubKey, Predictable, 1);
+
+        let mut dna_store = MockDnaStore::new();
+        dna_store.expect_add_dnas::<Vec<_>>().return_const(());
+        dna_store.expect_add_entry_defs::<Vec<_>>().return_const(());
+        dna_store.expect_add_dna().return_const(());
+        dna_store
+            .expect_get()
+            .return_const(Some(dna_file.clone().into()));
+
+        let mut conductor =
+            SweetConductor::from_builder(ConductorBuilder::with_mock_dna_store(dna_store)).await;
+
+        let apps = conductor
+            .setup_app_for_agents(
+                "app-",
+                &[alice_pubkey.clone(), bob_pubkey],
+                &[dna_file.into()],
+            )
             .await
             .unwrap();
 
-        let mut host_access = fixt!(ZomeCallHostAccess, Predictable);
-        host_access.workspace = workspace;
+        let ((alice,), (_bob,)) = apps.into_tuples();
+        let alice = alice.zome(TestWasm::HdkExtern);
 
-        let foo_result: String =
-            crate::call_test_ribosome!(host_access, TestWasm::HdkExtern, "foo", ()).unwrap();
+        let foo_result: String = conductor.call(&alice, "foo", ()).await;
 
-        assert_eq!("foo", foo_result.as_str());
+        assert_eq!("foo", &foo_result);
 
-        let bar_result: String =
-            crate::call_test_ribosome!(host_access, TestWasm::HdkExtern, "bar", ()).unwrap();
+        let bar_result: String = conductor.call(&alice, "bar", ()).await;
 
-        assert_eq!("foobar", bar_result.as_str());
+        assert_eq!("foobar", &bar_result);
+
+        let infallible_result = conductor
+            .handle()
+            .call_zome(ZomeCall {
+                cell_id: alice.cell_id().clone(),
+                zome_name: alice.name().clone(),
+                fn_name: "infallible".into(),
+                cap: None,
+                provenance: alice_pubkey.clone(),
+                payload: ExternIO::encode(()).unwrap(),
+            })
+            .await
+            .unwrap()
+            .unwrap();
+
+        if let ZomeCallResponse::Ok(response) = infallible_result {
+            assert_eq!("infallible", &response.decode::<String>().unwrap(),);
+        } else {
+            unreachable!();
+        }
     }
 }
